import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;

/**
 * Clase Circulo que es un extends de la clase Forma
 * @author Mauricio Solis y Cesar Munoz
*/
public class Circulo extends Forma
{
   /**
   *Costructor:    inicializa un nuevo Circulo
   *@param radio   radio del Circulo
   *@param center  centro del circulo
   */
   Circulo (Point center, double rad)
   {
      elipse = new Ellipse2D.Double(center.getX() - rad, center.getY(),
                                    2 * rad, 2 * rad);
      cuadro = new Cuadrado[ESQUINAS];
      for (int i = 0; i < ESQUINAS; i++)
      {
         cuadro[i] = new Cuadrado();
      }
      tipo = TYPE;
   }
   
   /**Metodo utilizado para reposicionar los cuadritos
    * de manipulacion
    */
   public void reposicionarCuadro()
   {
   double xx;
   double yy;
   double w, h;
   xx = elipse.getX();
   yy = elipse.getY();
   w  = elipse.getWidth();
   h  = elipse.getHeight();
   
   cuadro[0].setPoint(xx + w, yy + h / 2); //cuadrado derecho
   cuadro[1].setPoint(xx + w / 2, yy);     //cuadrado arriva
   cuadro[2].setPoint(xx, yy + h / 2);     //cuadrado izquierdo
   cuadro[ESQUINAS - 1].setPoint(xx + w / 2, yy + h); //cuadrado abajo
   }
   
   /**
   *Metodo que permite dibujar la figura en pantalla 
   *@param g2d Variable Graphics2D
   */
   public void paintComponent(Graphics2D g2d)
   {
      g2d.draw(elipse);
   }
  
   /**
   * Permite cambiar el radio del Circulo
   * @param r valor del radio 
   */
   public void setRadio(double r)
   {
      double centerX = elipse.getX() + elipse.getWidth() / 2;
      double centerY = elipse.getY() + elipse.getHeight() / 2;
      elipse.setFrame(centerX - r, centerY - r, 2 * r, 2 * r);
      reposicionarCuadro();
      
   }

   /**
   * Permite cambiar el tamano en forma escalada
   * @param c Punto respecto al cual se realiza el escalamiento
   * @param factor factor del escalamiento
   */
   public void escalar (Point c, double factor)
   {
      double x = elipse.getX();
      double y = elipse.getY();
      x = c.getX() + factor * (x - c.getX());
      y = c.getY() + factor * (y - c.getY());
      double diameter = elipse.getWidth() * factor;
      elipse.setFrame(x, y, diameter, diameter);
      reposicionarCuadro();
   }
    
   /**
   * Permite rotar la figura respecto a algun punto
   * @param c Punto respecto al que se rotara la figura
   * @param deltaAngulo angulo de rotacion
   */
   public void rotar (Point c, double deltaAngulo)
   {
      double centerX        = elipse.getX() + elipse.getWidth()  / 2 - c.getX();
      double centerY        = elipse.getY() + elipse.getHeight() / 2 - c.getY();
      double distancia      = Math.sqrt(centerX * centerX + centerY * centerY);
      double anguloInicial  = Math.atan2(centerY, centerX);
      centerX = distancia * Math.cos(anguloInicial + deltaAngulo) + c.getX();
      centerY = distancia * Math.sin(anguloInicial + deltaAngulo) + c.getY();
      double diameter = elipse.getWidth();
      elipse.setFrame(centerX - diameter / 2, centerY - diameter / 2,
                      diameter, diameter);
      reposicionarCuadro();
   }

   /**
   * Permite trasladar la figura dentro de la ventana
   * @param dx desplazamiento en el eje x de la figura
   * @param dy desplazamiento en el eje y de la figura
   */
   public void trasladar (double dx, double dy)
   {
      double x = elipse.getX() + dx;
      double y = elipse.getY() + dy;
      double diameter = elipse.getWidth();
      elipse.setFrame(x, y, diameter, diameter);
      reposicionarCuadro();
   }
 
   /**
   * Devuelve el tipo de figura
   * @return Retorna un entero, que corresponde al tipo de figura actual
   */
   public int getTipoFigura()
   {
      return tipo;
   }
 
   /**
   * Devuelve uno de los cuadritos de manipulacion la figura
   * @param i numero del cuadrito de manipulacion que se desea obtener
   * @return Retorna un cuadrado
   */
   public Cuadrado getVertice(int i)
   {
      return cuadro[i];
   }

   /**
    * Retorna el punto con respecto al cual se modificara el circulo
    * @param vertice de la figura a modificar
    * @return punto de retorno
    */
   public Point getRef(Point vertice)
   {
   for (int i = 0; i < ESQUINAS; i++)
   {
      if (cuadro[i].pertenece(vertice))
      {      
      return cuadro[(i + 2) % ESQUINAS].getCorner();
      }
   }
   Point p = new Point();
   return p;
   }
 
   private Ellipse2D.Double elipse;
   private int tipo;
   private Cuadrado[] cuadro;
   private static final int TYPE        = 2;
   private static final int ESQUINAS    = 4;
}
