import java.awt.event.*;
import java.awt.*;

/**
 * Manjeador para crear el circulo a partir de los 
 * eventos del mouse en el panel
 */
class CrearCirculoHandler implements ActionListener
{
   private MiPanel mipanel;
   private Circulo circulo;
   private int estado;
   private Point centro;
   private static final int INICIAL = 0;
   private static final int AFTER_CREADO = 1;
    
   /**
    * Constructor de la clase
    * @param dib corresponde al panel donde se dibujara la forma
    */
   public CrearCirculoHandler(MiPanel dib) 
   {
   mipanel = dib;
   estado  = INICIAL;  
   }

   /**
    * Espera por el evento de cliquear sobre el boton
    * @param e corresponde al evento
    */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      estado = INICIAL;
      mipanel.borrarCuadrados();
      mipanel.borrarCruces();
      mipanel.removerSelector();
   }

   /**
    * Manejador del mouse para crear el circulo
    */
   class MouseHandler extends MouseAdapter
   {
      /**
       * Evento de presionar el mouse
       * @param event evento de mouse
       */
      public void mousePressed(MouseEvent event)
      {
         if (estado == INICIAL)
         {
           centro = event.getPoint();
           circulo = new Circulo(centro, 0);
           mipanel.incorpore(circulo);
           estado = AFTER_CREADO;
           mipanel.repaint();
         }
      }
      
       /**
        * Evento de soltar el mouse
        * @param event evento de mouse
        */
      public void mouseReleased(MouseEvent event)
      {
         estado = INICIAL;
      }
   }
  
   /**
    * Manejador del mouse para crear el circulo
    */
   class MouseMotionHandler extends MouseMotionAdapter
   {
/**
     * Evento de arratrar el mouse
     * @param event evento de mouse
     */
      public void mouseDragged(MouseEvent event)
      {
         if (estado == AFTER_CREADO)
         {
            Point p = event.getPoint();
            double a = p.getX() - centro.getX();
            double b = p.getY() - centro.getY();
            double newRadio = Math.sqrt(a * a + b * b);
            circulo.setRadio(newRadio);
            mipanel.repaint();
         }  
      }
   }  
}
