import java.awt.event.*;
import java.awt.*;

/**
 * Manjeador para crear una linea en forma grafica
 */
class CrearLineaHandler implements ActionListener
{
   private MiPanel mipanel;
   private Linea linea;
   private int estado;
   private Point firstCorner;
   private static final int INICIAL = 0;
   private static final int AFTER_CREADO = 1;
  
   /**
    * Constructor de la Linea
    * @param dib corresponde al panel donde se dibujara la forma
    */
   public CrearLineaHandler(MiPanel dib)
   {
      mipanel = dib;
      estado  = INICIAL;
   }
  
   /**
    * Espera por el evento de cliquear sobre el boton
    * @param e corresponde al evento
    */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      estado = INICIAL;
      mipanel.borrarCuadrados();
      mipanel.borrarCruces();
      mipanel.removerSelector();
   }
  
   /**
    * Manejador del mouse para crear una linea
    */
   class MouseHandler extends MouseAdapter
   {
      /**
       * Evento de presionar el mouse
       * @param event evento de mouse
       */
      public void mousePressed(MouseEvent event)
      {
         if (estado == INICIAL)
         {
            firstCorner = event.getPoint();
            linea = new Linea(firstCorner, 0);
            mipanel.incorpore(linea);
            estado = AFTER_CREADO;
            mipanel.repaint();
         }   
      }
      /**
       * Evento de soltar el mouse
       * @param event evento de mouse
       */
      public void mouseReleased(MouseEvent event)
      {
         estado = INICIAL;
      }
   }
  
   /**
    * Manejador del mouse para crear una linea
    */
   class MouseMotionHandler extends MouseMotionAdapter
   {
      /**
      * Evento de arratrar el mouse
      * @param event evento de mouse
      */
      public void mouseDragged(MouseEvent event)
      {
         if (estado == AFTER_CREADO)
         {
            Point p = event.getPoint();
            linea.setOposite(p);
            mipanel.repaint();
         }  
      }
   }  
}
