import java.awt.event.*;
import java.awt.*;

/**
 * Manjeador para crear el rectangulo en forma grafica
 */
class CrearRectanguloHandler implements ActionListener
{
   private MiPanel mipanel;
   private Rectangulo rectangulo;
   private int estado;
   private Point firstCorner;
   private static final int INICIAL = 0;
   private static final int AFTER_CREADO = 1;
  
   /**
    * Constructor de la clase
    * @param dib corresponde al panel donde se dibujara la forma
    */
   public CrearRectanguloHandler(MiPanel dib)
   {
      mipanel = dib;
      estado  = INICIAL;
   }
  
   /**
    * espera por un evento en el boton de menu
    * @param e corresponde al evento
    */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      estado = INICIAL;
      mipanel.borrarCuadrados();
      mipanel.borrarCruces();
      mipanel.removerSelector();
   }
  
   /**
    * Manejador del mouse para crear el rectangulo
    */
   class MouseHandler extends MouseAdapter
   {
      /**
      * Evento de presionar el mouse
      * @param event evento de mouse
      */
      public void mousePressed(MouseEvent event)
      {
         if (estado == INICIAL)
         {
            firstCorner = event.getPoint();
            rectangulo = new Rectangulo(firstCorner, 0, 0);
            mipanel.incorpore(rectangulo);
            estado = AFTER_CREADO;
            mipanel.repaint();
         }
      }
      /**
       * Evento de soltar el mouse
       * @param event evento de mouse
       */
      public void mouseReleased(MouseEvent event)
      {
        estado = INICIAL;
      }
   }
  
   /**
    * Manejador del mouse para crear el rectangulo
    */
   class MouseMotionHandler extends MouseMotionAdapter
   {
   
   /**
    * Evento de arratrar el mouse
    * @param event evento de mouse
    */
      public void mouseDragged(MouseEvent event)
      {
         if (estado == AFTER_CREADO)
         {
            Point p = event.getPoint();
            rectangulo.setThirdCorner(p);
            mipanel.repaint();
         }
      }
   }  
}


