import java.awt.event.*;
import java.awt.*;

/**
 * Manjeador para crear un triangulo en forma grafica
 */
class CrearTrianguloHandler implements ActionListener
{
   private MiPanel mipanel;
   private Triangulo triangulo;
   private int estado;
   private Point firstCorner;
   private Point secondCorner;
   private Point thirdCorner;
   private static final int INICIAL       = 0;
   private static final int SEGUNDO_PUNTO = 2;
   private static final int TERCER_PUNTO  = 3;
   private static final int AFTER_CREADO  = 4;
  
   /**
    * Constructor de la clase
    * @param dib corresponde al panel donde se dibujara la forma
    */
   public CrearTrianguloHandler(MiPanel dib)
   {
      mipanel = dib;
      estado  = INICIAL;
   }
  
   /**
    * espera por elliquear sobre el boton
    * @param e corresponde al evento
    */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      estado = INICIAL;
      mipanel.borrarCuadrados();
      mipanel.borrarCruces();
      mipanel.removerSelector();
   }
  
   /**
    * Manejador del mouse para crear un triangulo
    */
   class MouseHandler extends MouseAdapter
   {
      /**
       * Evento de presionar el mouse
       * @param event evento de mouse
       */
      public void mousePressed(MouseEvent event)
      {
         if (estado == INICIAL)
         {
            firstCorner = event.getPoint();
            triangulo = new Triangulo(firstCorner, firstCorner, firstCorner);
            mipanel.incorpore(triangulo);
            mipanel.repaint();
            estado = SEGUNDO_PUNTO;
         }

         if (estado == TERCER_PUNTO)
         {
            thirdCorner = event.getPoint();
            triangulo.set3Corner(thirdCorner);
            mipanel.repaint();
            estado = AFTER_CREADO;
         }
      }
      /**
       * Evento de soltar el mouse
       * @param event evento de mouse
       */
      public void mouseReleased(MouseEvent event)
      {
         if (estado == SEGUNDO_PUNTO)
         {
            secondCorner = event.getPoint();
            mipanel.repaint();
            estado = TERCER_PUNTO;
         }
        
         if (estado == AFTER_CREADO)
         {
            thirdCorner = event.getPoint();            
            mipanel.repaint();
            estado = INICIAL;
         }
      }
   }
  
   /**
    * Manejador del mouse para crear un triangulo
    */
   class MouseMotionHandler extends MouseMotionAdapter
   {
      /**
       * Evento de arratrar el mouse
       * @param event evento de mouse
       */
      public void mouseDragged(MouseEvent event)
      {
         if (estado == SEGUNDO_PUNTO)
         {
            secondCorner = event.getPoint();
            triangulo.set2Corner(secondCorner);
            mipanel.repaint();
         }
        
         if (estado == AFTER_CREADO)
         {
            thirdCorner = event.getPoint();
            triangulo.set3Corner(thirdCorner);
            mipanel.repaint();
         }
      }
   }  
}
