import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;

/**
 * Clase Cruz que es un extends de la clase Forma
 * @author Mauricio Solis y Cesar Munoz
 */
public class Cruz extends Forma
{
   /**
    * Costructor:    inicializa una nueva Cruz
    */
   Cruz ()
   {
      center = new Point();
      size = SIZE_CRUZ;
      tipo = TYPE;
   }

   /**
    * Setea el punto central de la cruz
    * @param c punto centro de la cruz
    */
   public void setPoint(Point c)
   {
      center = new Point(c);
   }

   /**
    * Metodo que permite dibujar la figura en pantalla
    * @param g2d Variable Graphics2d
    */
   public void paintComponent(Graphics2D g2d)
   {
      g2d.draw(new Line2D.Double(center.getX() - size , center.getY() - size,
                                 center.getX() + size , center.getY() + size));
      g2d.draw(new Line2D.Double(center.getX() + size , center.getY() - size,
                                 center.getX() - size , center.getY() + size));
   }
   
   /**
    * Permite cambiar el tamano en forma escalada
    * @param c Punto respecto al cual se realiza el escalamiento
    * @param factor factor del escalamiento
    */
   public void escalar (Point c, double factor)
   {
      if (!center.equals(c))
      {
         center.translate((int)(factor * (center.getX() - c.getX())),
                        (int) (factor * (center.getY() - c.getY())));
         size *= factor;
      }
   }
   
   /**
    * Permite rotar la figura respecto a algun punto
    * @param c Punto respecto al que se rotara la figura
    * @param deltaAngulo angulo de rotacion
    */
   public void rotar (Point c, double deltaAngulo)
   {
      if (!center.equals(c))
      {
         double x = center.getX() - c.getX();
         double y = center.getY() - c.getY();
         double distancia = Math.sqrt(x * x + y * y);
         double anguloInicial = Math.atan2(y, x);
         x = distancia * Math.cos(anguloInicial + deltaAngulo);
         y = distancia * Math.sin(anguloInicial + deltaAngulo);
         center.setLocation(c.getX() + x, c.getY() + y);
      }
   }
   
   /**
    * Permite trasladar la figura dentro de la ventana
    * @param dx desplazamiento en el eje x de la figura
    * @param dy desplazamiento en el eje y de la figura
    */
   public void trasladar (double dx, double dy)
   {
      center.translate((int)dx, (int) dy);
   }
   
   /**
    * Devuelve el tipo de figura
    * @return Retorna un entero, que corresponde al tipo de figura actual
    */    
   public int getTipoFigura()
   {
      return tipo;
   }
   
   /**
    * Metodo heredado de la clase forma, pero que no
    * es utilizdo por esta clase, pero que debe retornar 
    * un punto
    * @param vertice vertice de la figura de la que
    * retornara un punto
    * @return retorna un punto
    */
   public Point getRef(Point vertice)
   {
      Point p = new Point();
      return p;
   }
    
   private Point center;
   private int size;
   private int tipo;
   private static final int SIZE_CRUZ = 5;
   private static final int TYPE      = 6;   
}
