import java.awt.*;
import java.awt.geom.*;
//import java.lang.Math;

/**
 * Clase Cuadrado que es un extends de la clase Rectangle2D.Double
 * @author Mauricio Solis y Cesar Munoz
 */

public class Cuadrado extends Rectangle2D.Double
{
   private static final int TAMANO         = 6;
   private static final int FORMA_ID       = 0;
   private static final int BORDER_SIZE    = 6;
   private static final int ESQUINAS = 3; 
  
 
   /**
    * Constructor:    inicializa un nuevo Cuadrado
    */
   Cuadrado ()
   {
      super();
      corner  = new Point();
      size    = TAMANO;
      formaId = FORMA_ID;
      pintar  = false;
   }
   
   /**
    * Constructor:    inicializa un nuevo Cuadrado
    * @param x indica el valor en x para la esquina superior izquierda
    * @param y indica el valor en y para la esquina superior izquierda
    */
   Cuadrado (double x, double y)
   {
      super(x - (TAMANO / 2), y - TAMANO / 2, BORDER_SIZE, BORDER_SIZE);
      corner = new Point((int) x, (int) y);
      size = BORDER_SIZE;       
      formaId = FORMA_ID;
      corner.setLocation(super.getX(), super.getY());
      pintar = false;
   }
   
   /**
    * Setea el punto central del Cuadrado
    * @param x indica el valor en x para el punto a setear
    * @param y indica el valor en x para el punto a setear
    */  
   public void setPoint(double x, double y)
   {
      this.setRect(x - (TAMANO / 2), y - (TAMANO / 2), size, size);
      corner.setLocation((int) x - (TAMANO / 2), (int) y - (TAMANO / 2));
   }
   
   /**
    * Setea el punto del Centro del Cuadrado 
    * @param c punto centro de lal Cuadrado
    */   
   public void setPoint(Point c)
   {
      super.setRect(c.getX(), c.getY(), size, size);    
      corner.setLocation(c.getX() - ESQUINAS, c.getY() - ESQUINAS);    
   }
   
   /**
    * Metodo que permite dibujar la figura en pantalla
    * @param g2d Variable Graphics2d
    */
   public void paintComponent(Graphics2D g2d)
   {
      if (pintar)
      {
         g2d.setColor(Color.black);
         g2d.draw(this);
         g2d.fill((Rectangle2D.Double)this);
  }
  else
  {
         g2d.setColor(Color.black);
         g2d.draw(this);
  }
   }

   /**
    * Metodo para determinar si un punto esta contenido dentro
    * del Cuadrado;
    * @param p punto que se desea comprobar si pertenece al cuadrado
    * @return retorna verdadero o falso de acuerdo a si pertenece o no
    */
   public boolean pertenece(Point p)
   {
      double px = p.getX();
      double py = p.getY();
  
      if ((px < super.getX())
         || (px > super.getX() + size)
         || (py < super.getY())
         || (py > super.getY() + size)) return false;
     return true;  
   }

   /**
    * Setea en el cuadrito asociado a la forma, el indice que 
    * ocupa la forma en el arreglo de formas 
    * @param id tipo de forma asociada
    */
   public void setFormaId(int id)
   {
      formaId = id;
   }
  
   /**
    * Retorna el indice que ocupa la forma en el ArrayList
    * a la que esta asociado el cuadrito
    * @return retorna eindice
    */
   public int getFormaId()
   { 
      return formaId;
   }
   
   /**
    * Metodo que setea el el atributo que indica si el cuadrito
    * se pinta o no 
    * @param b valor del tipo boolean 
    */
   public void setPintar(boolean b)
   {
      pintar = b;
   }
   
   /**
    * Metodo que sirve para conseguir el punto sobre el cual se
    * ha construido el cuadrado
    * @return retorna el punto sobre el cual esta construido el 
    * cuadrdo
    */
   public Point getCorner()
   {
   return corner;
   }
 
   private boolean pintar;
   private Point corner;
   private int size;
   private int formaId;
}
