import java.awt.event.*;
import java.awt.*;
import java.lang.Math;

/**
 * Manejador de la rotacion
 */
class EscalarHandler implements ActionListener
{ 
   private static final int INICIO_NUEVO_ESCALAMIENTO = 0;
   private static final int FIGURA_SELECCIONADA = 2;
   private static final int ANCLA_FIJADA = 3;
   private static final int TERMINO_ESCALAMIENTO = 4;
   private static final int RECTANGULO_INICIAL = 5;
   private static final int AFTER_CREADO = 6;
   private static final int INDICE_FIG = -2;
   private static final int ESQUINAS = 3;
  
   private MiPanel mipanel;
   private Point firstCorner;
   private Point corredizo;
   private Point ancla;
   private Point select;
   private Cruz cruz;
   private Rectangulo selector;
  
   private double largoReferencia;
   private int estado;
   private int indiceFig;

   /**
   * Constructor de la clase
   * @param dib ventana donde se encuentran las formas a escalar
   */
   public EscalarHandler(MiPanel dib)
   {
      mipanel  = dib;
      cruz = new Cruz();
      indiceFig = INDICE_FIG;
   }
  
   /**
   * espera por un click en el boton del menu
   * @param e corresponde al evento
   */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      estado = INICIO_NUEVO_ESCALAMIENTO;
      mipanel.removerIndices();
      mipanel.dibujarCuadrados();
      mipanel.removerSelector();        
   }
  
   /**
   * Manejador de los eventos del mouse para la rotacion
   */
   class MouseHandler extends MouseAdapter
   { 
      /**
      * Evento de presionar el mouse
      * @param event evento de mouse
      */
      public void mousePressed (MouseEvent event)
      {
         select = event.getPoint();
      indiceFig = mipanel.puntoPertenece(select);
  
         if ((indiceFig < 0) && (estado == INICIO_NUEVO_ESCALAMIENTO))
         estado = RECTANGULO_INICIAL;
         if ((indiceFig >= 0) && (estado == INICIO_NUEVO_ESCALAMIENTO))
      {
      mipanel.paintCuadrados(indiceFig);
      mipanel.incorporarIndices(indiceFig);
      ancla = new Point(((Forma)(mipanel.getFigura(indiceFig))).getRef(select));
            mipanel.borrarCruces();
            cruz.setPoint(ancla);
            mipanel.incorpore(cruz);
   estado = ANCLA_FIJADA;
      }
         
      if (estado == ANCLA_FIJADA)
         {
            Point p = event.getPoint();
            double x = p.getX() - ancla.getX();
            double y = p.getY() - ancla.getY();
            largoReferencia = Math.sqrt(x * x + y * y);
          
            if (mipanel.getCantidadFormas() > 0)
            indiceFig = mipanel.puntoPertenece(p);
         }
     
         if (estado == RECTANGULO_INICIAL)
 {  
            firstCorner = new Point((int)((event.getPoint()).getX()),
            (int)((event.getPoint()).getY()));
            //se captura el primer punto del rectangulo Selector
            selector = new Rectangulo(firstCorner, 0, 0);
            selector.setEsSelector();
            mipanel.incorporarSelector(selector);
            mipanel.repaint();
            estado = AFTER_CREADO;
         } 
      }  
  
      /**
      * Evento de hacer click con el mouse
      * (captura un mousePressed - mouseRelease en el mismo punto)
      * @param event evento de mouse
      */
      public void mouseClicked(MouseEvent event)
      {
         if (estado == INICIO_NUEVO_ESCALAMIENTO)
         {
      mipanel.removerIndices();
   select = event.getPoint();
      if (indiceFig >= 0)//indiceFig fue seteado en mousePressed
      {
      mipanel.paintCuadrados(indiceFig);
      mipanel.repaint();
      mipanel.incorporarIndices(indiceFig);
      estado = FIGURA_SELECCIONADA;
            }
   else 
   {
      estado = INICIO_NUEVO_ESCALAMIENTO;
   }
         }
      
         else if (estado == FIGURA_SELECCIONADA)
      {
      select = event.getPoint();   
   ancla = select;
               mipanel.borrarCruces();
               cruz.setPoint(ancla);
               mipanel.incorpore(cruz);
      estado = ANCLA_FIJADA;
            }  
      } 
      
      /**
      * Evento de soltar el mouse
      * @param event evento de mouse
      */
      public void mouseReleased(MouseEvent event)
      {
      if (estado == ANCLA_FIJADA)
         {
            mipanel.borrarCruces();
         mipanel.borrarCuadrados();
   mipanel.removerSelector();
            estado = TERMINO_ESCALAMIENTO;
         }  
  
         if (estado == AFTER_CREADO) 
         { 
         if (mipanel.cantidadFigurasManipular() > 0)
         {
               ancla = selector.getCentro();
               mipanel.borrarCruces();
               cruz.setPoint(ancla);
               mipanel.incorpore(cruz);
               mipanel.repaint();
               estado = ANCLA_FIJADA;
          }
         else //no se selecciono ninguna figura
   {
   estado = TERMINO_ESCALAMIENTO;
   mipanel.removerSelector();
   mipanel.borrarCuadrados();
   }
   }         
      }
   }  
   
   /**
   * Manejador del mouse para evetos "continuos" del mouse
   */
   class MouseMotionHandler extends MouseMotionAdapter
   {  /**
      * Evento de arratrar el mouse
      * @param event evento de mouse
      */
      public void mouseDragged(MouseEvent event)
      {  if (estado == ANCLA_FIJADA)
         {  Point p = event.getPoint();
            double x = p.getX() - ancla.getX();
            double y = p.getY() - ancla.getY();
            double distanciaAlAncla = Math.sqrt(x * x + y * y);
            double factorEscala = distanciaAlAncla / largoReferencia;
            if (indiceFig >= 0)
            { //incideFig fue seteado al momento de seleccionar la figura
            for (int i = 0; i < mipanel.cantidadFigurasManipular(); i++)
       mipanel.escalar(ancla, factorEscala, mipanel.figuraManipular(i)); } 
       largoReferencia = distanciaAlAncla;
       mipanel.removerSelector(); }       
         if (estado == AFTER_CREADO)
         {  //indiceSelector = mipanel.getCantidadFormas();
            Point p = event.getPoint();
            corredizo = new Point();
            selector.setThirdCorner(p);
            mipanel.vaciarCuadrados(); //borrar relleno
            mipanel.removerIndices();
            int indSupX = (int)firstCorner.getX();
            int indSupY = (int)firstCorner.getY();
            int indInfX = (int)p.getX(), indInfY = (int)p.getY();
            if (indSupX < indInfX)
            { indInfX = indSupX; indSupX = (int)p.getX(); }
            if (indSupY < indInfY)
            { indInfY = indSupY; indSupY = (int)p.getY(); }   
for (int i = indInfX / ESQUINAS; i < indSupX / ESQUINAS; i++)
   for (int j = indInfY / ESQUINAS; j < indSupY / ESQUINAS; j++)
               {  corredizo.setLocation(ESQUINAS * i, ESQUINAS * j);
                  indiceFig = mipanel.puntoPertenece(corredizo);
                  if (indiceFig >= 0)
                  {
                  mipanel.paintCuadrados(indiceFig);
                  mipanel.repaint();
                  mipanel.incorporarIndices(indiceFig); }  
               }
            mipanel.repaint();  
         } 
      }
   }
}
