//import java.awt.geom.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;

/**
 * Clase MiPanel que es un extends de JPanel
 * @author Mauricio Solis y Cesar Munoz
 */
public class MiPanel extends JPanel
{
     /**
   * Costructor:    inicializa un nuevo panel
   */
   MiPanel()
   {
      super();
      formas = new ArrayList();
      formasCuadradas = new ArrayList();
      indices = new ArrayList();
      currentMouseListener = null;
      currentMouseMotionListener = null;
      escogedor = new ArrayList();
     }

   /**
    * Metodo que permite dibujar la figura en pantalla
    * @param g Variable Graphics
    */
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      Graphics2D g2d = (Graphics2D) g;

      for (int i = 0; i < formas.size(); i++)
         ((Forma)formas.get(i)).paintComponent(g2d);

      if (escogedor.size() > 0)
      ((Rectangulo)escogedor.get(0)).paintComponent(g2d);

      for (int i = 0; i < formasCuadradas.size(); i++)
         ((Cuadrado)formasCuadradas.get(i)).paintComponent(g2d);
   }

   /**
   * Manejador de los eventos del mouse
   * @param l evento de mouse
   */
   public void changeMouseListener(MouseListener l)
   {
      if (currentMouseListener != null)
      removeMouseListener(currentMouseListener);
      currentMouseListener = l;
      addMouseListener(currentMouseListener);
   }

   /**
   * Manejador del mouse
   * @param l evento de mouse
   */
   public void changeMouseMotionListener(MouseMotionListener l)
   {
      if (currentMouseMotionListener != null)
      removeMouseMotionListener(currentMouseMotionListener);
      currentMouseMotionListener = l;
      addMouseMotionListener(currentMouseMotionListener);
   }

   /**
   * Permite cambiar el tamano en forma escalada
   * @param centro Punto respecto al cual se realiza el escalamiento
   * @param factor factor del escalamiento
   * @param i indice de la figura a escalar
   */

   public void escalar (Point centro, double factor, int i)
   {
      ((Forma)formas.get(i)).escalar(centro, factor);
      repaint();
   }

   /**
   * Permite rotar la figura respecto a algun punto
   * @param centro Punto respecto al que se rotara la figura
   * @param deltaAngulo angulo de rotacion
   * @param i indice de la figura a rotar
   */
   public void rotar (Point centro, double deltaAngulo, int i)
   {
      ((Forma)formas.get(i)).rotar(centro, deltaAngulo);
      repaint();
   }

   /**
   * Permite trasladar la figura dentro de la ventana
   * @param dx desplazamiento en el eje x de la figura
   * @param dy desplazamiento en el eje y de la figura
   * @param i indice de la figura a trasladar
   */
   public void trasladar (double dx, double dy, int i)
   {
      ((Forma)formas.get(i)).trasladar(dx, dy);
      repaint();
   }

   /**
   * Remueve una figura
   * @param f numero de la figura a remover
   */
   public void remueva (int f)
   {
      formas.remove(f);
      repaint();
   }

   /**
    * Matedo para remover un cuadrado
    * @param i indice del cuadrado a remover
    */
   public void remuevaCuadrados(int i)
   {
      formasCuadradas.remove(i);
      repaint();
   }

   /**
   * Remueve una forma
   * @param f forma a remover
   */
   public void remueva (Forma f)
   {
      formas.remove(f);
      repaint();
   }

   /**
   * Incorpora una forma a la lista
   * @param f incorpora f a la lista
   */
   public void incorpore (Forma f)
   {
      formas.add(f);
      repaint();
   }

   /**
    * Metodo para incorporar un cuadrado a la lista de cuadrados
    * @param c cuadrado a incorporar
    */
   public void incorpore (Cuadrado c)
   {
      formasCuadradas.add(c);
      repaint();
   }

   /**
   * Retorna la cantidad de formas
   * @return Retorna la cantidad de formas
   */
   public int getCantidadFormas()
   {
      return formas.size();
   }

   /**
    * Metodo que retorna la cantidad de Cuadrados en el panel
    * @return retorna la cantidad de cuadrados dibujados
    */
   public int getCantidadCuadrados()
   {
      return formasCuadradas.size();
   }

   /**
   * Metodo para saber si un punto esta dentro de
   * los cuadros de manipulacion y el indice de la
   * figura que a que pertenece el cuadrito de manipulacion
   * @param previo punto a verificar
   * @return retorna el indice dentro del arreglo ocupado por
   * la figura a la que pertenece el cuadrito que contiene el punto
   */
   public int puntoPertenece(Point previo)
   {
      for (int i = 0; i < getCantidadCuadrados(); i++)
      {
         if (((Cuadrado)getFormasCuadradas(i)).pertenece(previo))
            return ((Cuadrado)getFormasCuadradas(i)).getFormaId();
      }
         return -1;
   }

   /**
   * Retorna una figura
   * @param i figura a obtener
   * @return forma retornada
   */
   public Forma getFigura(int i)
   {
      return (Forma)formas.get(i);
   }

   /**
    * Metodo para obtener un cuadrado que esta en el panel
    * @param i indice del cuadrado
    * @return retorna el cuadrado indicado por el indice
    */
   public Cuadrado getFormasCuadradas(int i)
   {
      return (Cuadrado)formasCuadradas.get(i);
   }

   /**
    * Metodo para dibujar los cuadrados en el panel
    */
   public void dibujarCuadrados()
   {
      if (getCantidadFormas() > 0)
      {
         borrarCuadrados();
           int ff = 0;
         for (int i = 0; i < getCantidadFormas(); i++)
         {
            ff = (getFigura(i)).getTipoFigura();
            switch (ff)
               {
                 case 1: for (int j = 0; j < DOS_LADO; j++)
                         { incorpore((Cuadrado)
                           ((Linea)getFigura(i)).getVertice(j));
                           ((Cuadrado)((Linea)getFigura(i)).
                           getVertice(j)).setFormaId(i);
                         } break;
                case 2: for (int j = 0; j < CUATRO_LADO; j++)
                         { incorpore((Cuadrado)
                           ((Circulo)getFigura(i)).getVertice(j));
                           ((Cuadrado)((Circulo)getFigura(i)).
                           getVertice(j)).setFormaId(i);
                         } break;
                case 4: for (int j = 0; j < CUATRO_LADO; j++)
                         { incorpore((Cuadrado)
                           ((Rectangulo)getFigura(i)).getVertice(j));
                           ((Cuadrado)((Rectangulo)getFigura(i)).
                           getVertice(j)).setFormaId(i);
                         } break;
                case 5: for (int j = 0; j < TRES_LADO; j++)
                         { incorpore((Cuadrado)
                           ((Triangulo)getFigura(i)).getVertice(j));
                           ((Cuadrado)((Triangulo)getFigura(i)).
                           getVertice(j)).setFormaId(i);
                         } break;
                default: break;
               }
         }
      }
   }

   /**
    * Metodo que borra todos los cuadrados del panel
    */
   public void borrarCuadrados()
   {
      for (int i = getCantidadCuadrados() - 1; i >= 0; i--)
     //volver el color trasnparanete
       {
         ((Cuadrado)getFormasCuadradas(i)).setPintar(false);
       }
       int cantFig = getCantidadCuadrados();
       if (cantFig > 0)
       for (int i = cantFig - 1; i >= 0; i--)
         remuevaCuadrados(i);
   }

   /**
    *Metodo para borrar todas las cruces del panel
    */
   public void borrarCruces()
   {
      if (getCantidadFormas() > 0)
         for (int i = getCantidadFormas() - 1; i >= 0; i--)
           {
            if ((getFigura(i)).getTipoFigura() == TYPE)remueva(i);
          }
   }

   /**
    * Metodo para pintar (de color) los cuadritos
    * @param ind corresponde al indice de la figura a la cual
    * se le quieren pintar los cuadritos
    */
   public void paintCuadrados(int ind)//indice de la figura
   {
      if (getCantidadFormas() > 0)
         for (int k = getCantidadCuadrados() - 1; k >= 0; k--)
            if (((Cuadrado)getFormasCuadradas(k)).getFormaId() == ind)
               ((Cuadrado)getFormasCuadradas(k)).setPintar(true);
   }

   /**
    * Metodo para despintar los cuadritos de las figuras
    */
   public void vaciarCuadrados()
   {
      for (int k = getCantidadCuadrados() - 1; k >= 0; k--)
         ((Cuadrado)getFormasCuadradas(k)).setPintar(false);
   }

   /**
    * Metodo para incorporar los indices de lo las figuras
    * que se desea pintar los cuadritos
    * @param ind indice de la figura a pintar
    */
   public void incorporarIndices(int ind)
   {
      boolean verificador = false;
      int i = 0;
      int figAmanipular = -1;
      while ((i < cantidadFigurasManipular()) && (!verificador))
      {
         figAmanipular = figuraManipular(i);
         if (ind == figAmanipular) verificador = true;
         i += 1;
      }

      if (!verificador)indices.add(new Integer(ind));
   }

   /**
    * Metodo para conocer la cantidad de figuras a manipular
    * @return retorna el tamano del arreglo de indices
    */
   public int cantidadFigurasManipular()
   {
      return indices.size();
   }

   /**
    * Metodo para conseguir el indice de la figura a manipular
    * @param ind indice del arreglo de indices de las figuras
    * @return el indice de la figura a manipular
    */
   public int figuraManipular(int ind)
   {
      return ((Integer)indices.get(ind)).intValue();
   }

   /**
    * Metodo para remover todos los indices del
    * arreglo de indices (elimina los indices de
    * las figuras a manipular)
    */
   public void removerIndices()
   {
      for (int i = cantidadFigurasManipular() - 1; i >= 0; i--)
         indices.remove(i);
   }

   /**
    * Metodo para incorporar al panel el rectangulo
    * seleccionador de figuras
    * @param r rectangulo a incorporar
    */
   public void incorporarSelector(Rectangulo r)
   {
      escogedor.add(r);
   }

   /**
    * Metodo para remover el Rectangulo selector
    */
   public void removerSelector()
   {
      for (int i = escogedor.size() - 1; i >= 0; i--)
      escogedor.remove(i);
   }
 /**
    * Metodo para remover el Rectangulo selector
    * @param punto Remueve el rectangulo selector
    * @return retorna la forma a eliminar
    */
   public int getCuadradoId(Point punto)
   {
      for (int i = 0; i < getCantidadCuadrados(); i++)
       if (((Cuadrado)getFormasCuadradas(i)).pertenece(punto)) return (i);
      return -1;
   }

   private static final int DOS_LADO     = 2;
   private static final int TRES_LADO    = 3;
   private static final int CUATRO_LADO  = 4;
   private static final int TYPE         = 6;
   private ArrayList indices;
   private ArrayList formas;
   private ArrayList formasCuadradas;
   private ArrayList escogedor;
   private MouseListener currentMouseListener;
   private MouseMotionListener currentMouseMotionListener;
}
