import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;

/**
 * Clase Rectangulo que es un extends de la clase Forma
 * @author Mauricio Solis y Cesar Munoz
 */
public class Rectangulo extends Forma
{
   private Point2D[] corner;
   private Cuadrado[] cuadro;
   private int tipo;
   private boolean esSelector;
   private static final int ESQUINAS    = 4;
   private static final int TYPE        = 4;
   private static final int ESQUINA3 = 3;

   /**
    * Clase Rectangulo que es un extends de la clase Forma
    * @param p punto inicial, primer vertice
    * @param w ancho del rectangulo
    * @param h alto del rectangulo
    */
   Rectangulo (Point p, double w, double h)
   {
      corner = new Point2D[ESQUINAS];
      cuadro = new Cuadrado[ESQUINAS];
      corner[0] = new Point2D.Double(p.getX(), p.getY());
      corner[1] = new Point2D.Double(p.getX(), p.getY() + h);
      corner[2] = new Point2D.Double(p.getX() + w, p.getY() + h);

      corner[ESQUINAS - 1] = new Point2D.Double(p.getX() + w, p.getY());

      for (int i = 0; i < ESQUINAS; i++)
      {
         cuadro[i] = new Cuadrado();
         cuadro[i].setPoint(corner[i].getX(), corner[i].getY());
      }

      tipo = TYPE;
      esSelector = false;
   }

   /**
    *Metodo que permite dibujar la figura en pantalla
    *@param g2d Variable Graphics2d
    */
   public void paintComponent(Graphics2D g2d)
   {
      if (esSelector) g2d.setColor(Color.blue);
      else g2d.setColor(Color.black);

      for (int i = 0; i < corner.length; i++)
      g2d.draw(new Line2D.Double(corner[i], corner[(i + 1) % corner.length]));
   }

   /**
    * Permite asignar los vertices
    * del Rectangulo en forma grafica
    * @param p punto al cual se setea el vertice opuesto
    */
   public void setThirdCorner(Point p)
   {
      corner[2] = new Point2D.Double(p.getX(), p.getY());
      corner[1].setLocation(corner[0].getX(), corner[2].getY());
      corner[ESQUINAS - 1].setLocation(corner[2].getX(), corner[0].getY());
      reposicionarCuadro();
   }

   /**
    * Permite cambiar el tamano en forma escalada
    * @param center Punto respecto al cual se realiza el escalamiento
    * @param factor factor del escalamiento
    */
   public void escalar(Point center, double factor)
   {
      for (int i = 0; i < corner.length; i++)
      {
         corner[i].setLocation(center.getX() + factor * (corner[i].getX()
         - center.getX()), center.getY() + factor * (corner[i].getY()
         - center.getY()));
      }
      reposicionarCuadro();
   }

   /**
    * Permite rotar la figura respecto a algun punto
    * @param center Punto respecto al que se rotara la figura
    * @param deltaAngulo angulo de rotacion
    */
   public void rotar(Point center, double deltaAngulo)
   {
      for (int i = 0; i < corner.length; i++)
      {
         double x = corner[i].getX() - center.getX();
         double y = corner[i].getY() - center.getY();
         double distancia = Math.sqrt(x * x + y * y);
         double anguloInicial = Math.atan2(y, x);
         x = distancia * Math.cos(anguloInicial + deltaAngulo) + center.getX();
         y = distancia * Math.sin(anguloInicial + deltaAngulo) + center.getY();
         corner[i].setLocation(x, y);
      }
      reposicionarCuadro();
   }

   /**
    * Permite trasladar la figura dentro de la ventana
    * @param dx desplazamiento en el eje x de la figura
    * @param dy desplazamiento en el eje y de la figura
    */
   public void trasladar(double dx, double dy)
   {
      for (int i = 0; i < corner.length; i++)
      corner[i].setLocation(corner[i].getX() + dx, corner[i].getY() + dy);
      reposicionarCuadro();
   }

   /**
    * reposiciona los cuadros de los vertices
    */
   public void reposicionarCuadro()
   {
      for (int i = 0; i < ESQUINAS; i++)
         cuadro[i].setPoint(corner[i].getX(), corner[i].getY());
   }

   /**
    * Devuelve uno de los vertices de la figura
    * @param i numero del vertice que se desea obtener
    * @return Retorna un cuadrado
    */
   public Cuadrado getVertice(int i)
   {
      return cuadro[i];
   }

   /**
    * Devuelve el tipo de figura
    * @return Retorna un entero, que corresponde al tipo de figura actual
    */
   public int getTipoFigura()
   {
      return tipo;
   }

   /**
    * Metodo encargado de calcular y retornar
    * el centro del rectangulo
    * @return retorna el punto del centro del rectangulo
    */
   public Point getCentro()
   {
      Point centro = new Point();
      double x1 = cuadro[0].getX();
      double x2 = cuadro[1].getX();
      double x3 = cuadro[2].getX();
      double x4 = cuadro[ESQUINA3].getX();
      double cx = 0.0, cy = 0.0;
      double y1 = cuadro[0].getY();
      double y2 = cuadro[1].getY();
      double y3 = cuadro[2].getY();
      double y4 = cuadro[ESQUINA3].getY();
      double r12 = Math.sqrt(Math.pow(x1 - x2, 2) + Math.pow(y1 - y2, 2));
      double r13 = Math.sqrt(Math.pow(x1 - x3, 2) + Math.pow(y1 - y3, 2));
      double r14 = Math.sqrt(Math.pow(x1 - x4, 2) + Math.pow(y1 - y4, 2));

      if ((r12 >= r13) && (r12 >= r14))
      {
         if (Math.abs(x1) >= Math.abs(x2))cx = x2 + (x1 - x2) / 2.0;
        else cx = x1 + (x2 - x1) / 2.0;

         if (Math.abs(y1) >= Math.abs(y2))cy = y2 + (y1 - y2) / 2.0;
         else cy = y1 + (y2 - y1) / 2.0;
      }
      if ((r13 >= r12) && (r13 >= r14))
      {
      if (Math.abs(x1) >= Math.abs(x3))cx = x3 + (x1 - x3) / 2.0;
      else cx = x1 + (x3 - x1) / 2.0;
      if (Math.abs(y1) >= Math.abs(y3))cy = y3 + (y1 - y3) / 2.0;
      else cy = y1 + (y3 - y1) / 2.0; }
      if ((r14 >= r12) && (r14 >= r13))
      {
         if ((Math.abs(x1) >= Math.abs(x4)))cx = x4 + (x1 - x4) / 2.0;
         else cx = x1 + (x4 - x1) / 2.0;
         if (Math.abs(y1) >= Math.abs(y3))cy = y4 + (y1 - y4) / 2.0;
         else cy = y1 + (y4 - y1) / 2.0;
      }

      centro.setLocation((int)cx, (int)cy);
      return centro;
   }

   /**
    * Metodo que permite definir si un rectangulo corresponde
    * al rectangulo selector de figuras o no
    *
    */
   public void setEsSelector()
{
    esSelector = true;
}

   /**
    * Metodo que permite obtener el punto con respecto
    * al que se realiara una modificacion
    * @param vertice    corresponde a algun punto que
    * pertenece a alguno de los cuadrados de manipulacion
    * de la figura
    * @return retorna el punto con respeco al que se debe
    * realizar la modificacion
    */
   public Point getRef(Point vertice)
   {
   for (int i = 0; i < ESQUINAS; i++)
   {
      if (cuadro[i].pertenece(vertice))
      {
      Point p1 = new Point();
      p1.setLocation((cuadro[(i + 2) % ESQUINAS]).getX() + 2,
       (cuadro[(i + 2) % ESQUINAS]).getY() + 2);
      return p1;
      }
   }
   Point p = new Point();
   return p;
   }
}
