import java.awt.event.*;
import java.awt.*;
//import java.awt.geom.*;
//import java.lang.Math;

/**
* Manejador del escalamiento
*/
class TrasladarHandler implements ActionListener
{
   private MiPanel mipanel;
   private Point select;
   private int indiceFig;
   private int estado;
   private Point firstCorner;
   private Rectangulo selector;
   private static final int VALOR_FIG    = -2;
   private static final int INICIO_NUEVA_TRASLACION = 0;
   private static final int TRASLADANDO = 1;
   private static final int FIGURA_SELECCIONADA = 2;
   private static final int TERMINO_TRASLACION = 4;
   private static final int RECTANGULO_INICIAL = 5;
   private static final int AFTER_CREADO = 6;
   private static final int ESQUINAS = 3;
   
   /**
   * Constructor de la clase
   * @param dib ventana donde se encuentran las formas a escalar
   */
   public TrasladarHandler(MiPanel dib)
   {
      mipanel = dib;
      indiceFig = VALOR_FIG;
   }
   
   /**
   * espera por el evento de cliquear sobre el boton
   * @param e corresponde al evento
   */
   public void actionPerformed(ActionEvent e)
   {
      mipanel.changeMouseListener(new MouseHandler());
      mipanel.changeMouseMotionListener(new MouseMotionHandler());
      mipanel.dibujarCuadrados();
      mipanel.removerIndices();
      mipanel.removerSelector();
      estado = INICIO_NUEVA_TRASLACION; 
   }  
   
   /**
   * Manejador del mouse para el escalamiento
   */
   class MouseHandler extends MouseAdapter
   {
      /**
      * Evento de presionar el mouse
      * @param event evento de mouse
      */
      public void mousePressed (MouseEvent event)
      {
         select = new Point(event.getPoint());
         indiceFig = mipanel.puntoPertenece(select);
         if ((indiceFig < 0) && (estado == INICIO_NUEVA_TRASLACION))
            estado = RECTANGULO_INICIAL;
         
         if ((indiceFig >= 0) && (estado == FIGURA_SELECCIONADA))
            estado =  TRASLADANDO;

         if ((indiceFig >= 0) && (estado == INICIO_NUEVA_TRASLACION))
         {
            mipanel.paintCuadrados(indiceFig);
            mipanel.repaint();
            mipanel.incorporarIndices(indiceFig);
            estado = TRASLADANDO;
         }
         if (estado == RECTANGULO_INICIAL)
         {  
         firstCorner = new Point((int)((event.getPoint()).getX()),
         (int)((event.getPoint()).getY()));
         //se captura el primer punto del rectangulo Selector
         selector = new Rectangulo(firstCorner, 0, 0);
         selector.setEsSelector();
         mipanel.incorporarSelector(selector);
         mipanel.repaint();
         estado = AFTER_CREADO;
         } 
      }
   /**
      * Evento de hacer click con el mouse
      * @param event evento de mouse
      */
      public void mouseClicked(MouseEvent event)
      {
         if (estado == INICIO_NUEVA_TRASLACION)
         {
            mipanel.removerIndices();
            select = event.getPoint();
            if (indiceFig >= 0)//indiceFig fue seteado en mousePressed
            {
            mipanel.paintCuadrados(indiceFig);
            mipanel.repaint();
            mipanel.incorporarIndices(indiceFig);
            estado = FIGURA_SELECCIONADA;
            }
         }
   }
      
      /**
      * Evento de soltar el mouse
      * @param event evento de mouse
      */  
      public void mouseReleased(MouseEvent event)
      {
      if (estado == AFTER_CREADO) 
      {
      if (mipanel.cantidadFigurasManipular() > 0)
      {
      mipanel.removerSelector();  
      estado = FIGURA_SELECCIONADA;
      mipanel.repaint();
            }
      else
      {
         estado = TERMINO_TRASLACION;
         mipanel.removerSelector();
         mipanel.borrarCuadrados();
       }
      }

      if (estado == TRASLADANDO)
      {
      estado = TERMINO_TRASLACION;
      mipanel.borrarCuadrados();
      } 
      }
   }
   
/**
   * Manejador del mouse para el escalamiento
   */ 
   class MouseMotionHandler extends MouseMotionAdapter
   { /**
      * Evento de arratrar el mouse
      * @param event evento de mouse
      */
      public void mouseDragged(MouseEvent event)
      {
      if (estado == TRASLADANDO)
         {  Point p = event.getPoint();
            double deltaX = p.getX() - select.getX();
            double deltaY = p.getY() - select.getY();
      for (int i = 0; i < mipanel.cantidadFigurasManipular(); i++)      
      mipanel.trasladar(deltaX, deltaY, mipanel.figuraManipular(i));
      select = p; }   
      if (estado == AFTER_CREADO)
         {  Point p = event.getPoint();
            Point corredizo = new Point();
            selector.setThirdCorner(p);
            mipanel.vaciarCuadrados(); //borrar relleno
            mipanel.removerIndices();
            int indSupX = (int)firstCorner.getX();
            int indSupY = (int)firstCorner.getY();
            int indInfX = (int)p.getX();
            int indInfY = (int)p.getY();
            if (indSupX < indInfX)
            {
            indInfX = indSupX;
            indSupX = (int)p.getX();
            }
            if (indSupY < indInfY)
             { indInfY = indSupY;
               indSupY = (int)p.getY(); }
               
            for (int i = indInfX / ESQUINAS; i < indSupX / ESQUINAS; i++)
            for (int j = indInfY / ESQUINAS; j < indSupY / ESQUINAS; j++)
            {  corredizo.setLocation(ESQUINAS * i, ESQUINAS * j);
               indiceFig = mipanel.puntoPertenece(corredizo);
               if (indiceFig >= 0)
               {   mipanel.paintCuadrados(indiceFig);
                   mipanel.repaint();
                  mipanel.incorporarIndices(indiceFig); }
            }
      mipanel.repaint();
       }
      }
   }
}
