import java.awt.*;
import java.awt.geom.*;
import java.lang.Math;

/**
 * Clase Triangulo que es un extends de la clase Forma
 * @author Mauricio Solis y Cesar Munoz
 */
public class Triangulo extends Forma
{   
   /**
     * Costructor:    inicializa un nuevo Triangulo
     * @param a Vertice a
     * @param b Vertice b
     * @param c Vertice c
     */
   Triangulo (Point a, Point b, Point c)
   {
      corner = new Point2D[ESQUINAS];
      cuadro = new Cuadrado[ESQUINAS];

      corner[0] = new Point2D.Double(a.getX(), a.getY());
      corner[1] = new Point2D.Double(b.getX(), b.getY());
      corner[2] = new Point2D.Double(c.getX(), c.getY());

      for (int i = 0; i < ESQUINAS; i++)
      {
          cuadro[i] = new Cuadrado();
          ((Cuadrado)cuadro[i]).setPoint(corner[i].getX(), corner[i].getY());
      }

      tipo = TYPE;
   }
   /**
    *Metodo que permite dibujar la figura en pantalla 
    *@param g2d Variable Graphics2d
    */
   public void paintComponent(Graphics2D g2d)
   {
     for (int i = 0; i < corner.length; i++)
      g2d.draw(new Line2D.Double(corner[i], corner[(i + 1) % corner.length]));
    }


   /**
    * Permite asignar los vertices
    * del traingulo en forma grafica
    * @param p punto al cual se setea el segundo vertice
    */
   public void set2Corner(Point p)
   {
      corner[2] = corner[0];
      corner[1] = new Point2D.Double(p.getX(), p.getY());
      reposicionarCuadro();
   }

   /**
    * Permite asignar los vertices
    * del traingulo en forma grafica
    * @param p punto al cual se setea el tercer vertice
    */
   public void set3Corner(Point p)
   {
      corner[2] = new Point2D.Double(p.getX(), p.getY());
      reposicionarCuadro();
   }

   /**
    * Permite cambiar el tamano en forma escalada
    * @param center Punto respecto al cual se realiza el escalamiento
    * @param factor factor del escalamiento
    */
   public void escalar(Point center, double factor)
   {
      for (int i = 0; i < corner.length; i++)
        corner[i].setLocation(center.getX() + factor
        * (corner[i].getX() - center.getX()), center.getY()
        + factor * (corner[i].getY() - center.getY()));
      reposicionarCuadro();
   }
   /**
    * Permite rotar la figura respecto a algun punto
    * @param center Punto respecto al que se rotara la figura
    * @param deltaAngulo angulo de rotacion
    */
   public void rotar(Point center, double deltaAngulo)
   {
      for (int i = 0; i < corner.length; i++)
      {
         double x = corner[i].getX() - center.getX();
         double y = corner[i].getY() - center.getY();
         double distancia = Math.sqrt(x * x + y * y);
         double anguloInicial = Math.atan2(y, x);
         x = distancia * Math.cos(anguloInicial + deltaAngulo) + center.getX();
         y = distancia * Math.sin(anguloInicial + deltaAngulo) + center.getY();
         corner[i].setLocation(x, y);
      }
      reposicionarCuadro();
   }
   /**
    * Permite trasladar la figura dentro de la ventana
    * @param dx desplazamiento en el eje x de la figura
    * @param dy desplazamiento en el eje y de la figura
    */
   public void trasladar(double dx, double dy)
   {
      for (int i = 0; i < corner.length; i++)
         corner[i].setLocation(corner[i].getX() + dx, corner[i].getY() + dy);
      reposicionarCuadro();
   }
 
   /**
    * Devuelve el tipo de figura
    * @return Retorna un entero, que corresponde al tipo de figura actual
    */
   public int getTipoFigura()
   {
      return tipo;
   }
   
   /**
    * reposiciona los cuadros de los vertices
    */
   public void reposicionarCuadro()
   {
      for (int i = 0; i < ESQUINAS; i++)
      ((Cuadrado)cuadro[i]).setPoint(corner[i].getX(), corner[i].getY());
   }
    
   /**
    * Devuelve uno de los vertices de la figura
    * @param i numero del vertice que se desea obtener
    * @return Retorna un cuadrado
    */
   public Cuadrado getVertice(int i)
   {
      return cuadro[i];
   }
   
   /**
    * Metodo que permite obtener el punto con respecto
    * al que se realiara una modificacion
    * @param vertice    corresponde a algun punto que 
    * pertenece a alguno de los cuadrados de manipulacion
    * de la figura
    * @return retorna el punto con respeco al que se debe 
    * realizar la modificacion
    */
   public Point getRef(Point vertice)
   {
   for (int i = 0; i < ESQUINAS; i++)
   {
      if (cuadro[i].pertenece(vertice))
    {
         Point p1 = new Point();
         p1.setLocation((cuadro[(i + 1) % ESQUINAS]).getX() + 2,
         (cuadro[(i + 1) % ESQUINAS]).getY() + 2);
         return p1;
     }
    }
        Point p = new Point();
        return p;
   }
   
  private Point2D[] corner;
  private int tipo;
  private Cuadrado[] cuadro;
  private static final int ESQUINAS = 3;
  private static final int TYPE     = 5;
}
